@@ -70,7 +70,9 @@ class ZaqarAlarmNotifier(notifier.AlarmNotifier):
                 z_srv = ks_client.services.find(
                     type=self.conf.service_types.zaqar)
                 endpoint_type = self.conf.service_credentials.interface
+                region_name = self.conf.service_credentials.region_name
                 z_endpoint = ks_client.endpoints.find(service_id=z_srv.id,
+                                                      region_id=region_name,
                                                       interface=endpoint_type)
                 self._zendpoint = z_endpoint.url
             except Exception:
@@ -159,11 +161,13 @@ class ZaqarAlarmNotifier(notifier.AlarmNotifier):
             # provide enough information about that. Otherwise, go to build
             # a client with service account and queue name for this alarm.
             conf, queue_name = self._get_presigned_client_conf(queue_info)
-            if conf is not None:
+
+            if conf is None:
+                zaqar_client = self.client
+            else:
                 zaqar_client = self.get_zaqar_client(conf)
 
-            if conf is None or queue_name is None or zaqar_client is None:
-                zaqar_client = self.client
+            if queue_name is None:
                 # queue_name is a combination of <alarm-id>-<topic>
                 queue_name = "%s-%s" % (message['body']['alarm_id'],
                                         queue_info.get('topic')[-1])
